/* Dacus
 * Modified by ElectroDacus (Dacian Todea) for the SBMS and DMPPT specific hardware
 *
 * see below the original license:
 *
  ******************************************************************************
  * @file    stm320518_eval_i2c_ee.h
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    20-April-2012
  * @brief   This file contains all the functions prototypes for 
  *          the stm320518_eval_i2c_ee.c firmware driver.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  *
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __DMPPT_I2C2_H
#define __DMPPT_I2C2_H

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "stm32f37x.h"
#include "stm32f37x_i2c.h"
#include "stm32f37x_gpio.h"
#include "stm32f37x_rcc.h"
#include "delay.h"


 // @brief  I2C Interface pins for DMPPT450

#define sDMPPT_I2C                          I2C2
#define sDMPPT_I2C_CLK                      RCC_APB1Periph_I2C2

#define sDMPPT_I2C_SCL_PIN                  GPIO_Pin_1                  /* PF.1 */
#define sDMPPT_I2C_SCL_GPIO_PORT            GPIOF                       /* GPIOF */
#define sDMPPT_I2C_SCL_GPIO_CLK             RCC_AHBPeriph_GPIOF
#define sDMPPT_I2C_SCL_SOURCE               GPIO_PinSource1
#define sDMPPT_I2C_SCL_AF                   GPIO_AF_4

#define sDMPPT_I2C_SDA_PIN                  GPIO_Pin_0                  /* PF.0 */
#define sDMPPT_I2C_SDA_GPIO_PORT            GPIOF                       /* GPIOF */
#define sDMPPT_I2C_SDA_GPIO_CLK             RCC_AHBPeriph_GPIOF
#define sDMPPT_I2C_SDA_SOURCE               GPIO_PinSource0
#define sDMPPT_I2C_SDA_AF                   GPIO_AF_4




 /* Select which I2C will be used with this driver */
 #define sDMPPT_1


#if defined (sDMPPT_1)
#define sDMPPT_HW_ADDRESS         0xA0           // 0x50 Address for BMS write and 0x51 for read
#define sDMPPT_I2C_TIMING         0x00951F21     // A0120227 for 16.6kHz  and 00951F21 for 100kHz
#define sDMPPT_PAGESIZE           50
#endif /* ISL94203 */





 /* Maximum Timeout values for flags and events waiting loops. These timeouts are
    not based on accurate values, they just guarantee that the application will
    not remain stuck if the I2C communication is corrupted.
    You may modify these timeout values depending on CPU frequency and application
    conditions (interrupts routines ...). */

 #define sDMPPT_FLAG_TIMEOUT         ((uint32_t)0x1000)
 #define sDMPPT_LONG_TIMEOUT         ((uint32_t)(10 * sDMPPT_FLAG_TIMEOUT))

 /* Maximum number of trials for sBMS_WaitEepromStandbyState() function */

 #define sDMPPT_MAX_TRIALS_NUMBER     50
 #define sDMPPT_OK                    0
 #define sDMPPT_FAIL                  1




 /**
   * @}
   */

  void sDMPPT_LowLevel_DeInit(void);
  void sDMPPT_LowLevel_Init(void);


 void     sDMPPT_DeInit(void);
 void     sDMPPT_Init(void);
 uint32_t sDMPPT_ReadBuffer(uint8_t* pBuffer, uint16_t ReadAddr, uint16_t* NumByteToRead);
 uint32_t sDMPPT_WritePage(uint8_t* pBuffer, uint16_t WriteAddr, uint8_t* NumByteToWrite);
 void     sDMPPT_WriteBuffer(uint8_t* pBuffer, uint16_t WriteAddr, uint16_t NumByteToWrite);
 uint32_t sDMPPT_WaitEepromStandbyState(void);

 /* USER Callbacks: These are functions for which prototypes only are declared in
    EEPROM driver and that should be implemented into user applicaiton. */
 /* sBMS_TIMEOUT_UserCallback() function is called whenever a timeout condition
    occure during communication (waiting on an event that doesn't occur, bus
    errors, busy devices ...).
    You can use the default timeout callback implementation by uncommenting the
    define USE_DEFAULT_TIMEOUT_CALLBACK in stm320518_evel_i2c_ee.h file.
    Typically the user implementation of this callback should reset I2C peripheral
    and re-initialize communication or in worst case reset all the application. */
 uint32_t sDMPPT_TIMEOUT_UserCallback(void);

 #ifdef __cplusplus
 }
 #endif

 #endif /* __STM320518_EVAL_I2C_EE_H */


 /************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
