/*
==========================================================================
    ElectroDacus Digital Max Power Point PV thermal controller DMPPT450 main file.
    v1.0c
    Copyright (C) 2017  <Dacian Todea>
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
===========================================================================
*/

#include "stm32f37x.h"
#include "stm32f37x_gpio.h"
#include "stm32f37x_rcc.h"
#include "stm32f37x_pwr.h"
#include "stm32f37x_rtc.h"
#include "stm32f37x_exti.h"
#include "stm32f37x_misc.h"
#include "stm32f37x_syscfg.h"
#include "stm32f37x_tim.h"
#include "stm32f37x_rcc.h"
#include "stm32f37x_usart.h"
#include "stm32f37x_adc.h"
#include "stm32f37x_iwdg.h"
#include <stdio.h>
#include "delay.h"
#include "defines.h"
#include "sdadc.h"
#include "DMPPT_i2c2.h"


//----------------------------------------------------------------------------------------------------------
#define DMPPT 450      //DMPPT450
#define test1 0        //DMPPT test
//----------------------------------------------------------------------------------------------------------




#if DMPPT==450
static unsigned int const softver=10;
static unsigned int const model=45;
static unsigned int const gain1=100;  //LMP8481
static float const shunt1=0.0001350; //TPV1,TPV6 8x1mOhm (added 8%)
static float const shunt2=0.0002650; //TPV2,TPV5,PVOUT1,PVOUT2 4x1mOhm (added 6%)
static float const shunt3=0.0001750; //TPV3,TPV4 6x1mOhm (added 5%)

static float const v30div=12.02; //divider for the DMPPTV
//static unsigned int const vopen=340;   // 34V
static unsigned int const vmppmax=315; // 31.5V
static unsigned int const vmppmin=285; // 28.5V
//static unsigned int const vmin=210;  // minimum acceptable PV array voltage
static unsigned int const amin=15; //minimum current for load detection 1.5A
static unsigned int const amax50=605;
static unsigned int const amax100=1050;

#endif


// Private defines
#define PIN_00 ((uint32_t)0)
#define PIN_01 ((uint32_t)1)
#define PIN_02 ((uint32_t)2)
#define PIN_03 ((uint32_t)3)
#define PIN_04 ((uint32_t)4)
#define PIN_05 ((uint32_t)5)
#define PIN_06 ((uint32_t)6)
#define PIN_07 ((uint32_t)7)
#define PIN_08 ((uint32_t)8)
#define PIN_09 ((uint32_t)9)
#define PIN_10 ((uint32_t)10)
#define PIN_11 ((uint32_t)11)
#define PIN_12 ((uint32_t)12)
#define PIN_13 ((uint32_t)13)
#define PIN_14 ((uint32_t)14)
#define PIN_15 ((uint32_t)15)

// LEDs definition
#define LD1_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_03);}
#define LD1_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_03 + 16));}
#define LD2_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_02);}
#define LD2_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_02 + 16));}
#define LD3_ON      {GPIOC->BSRR = (uint32_t)((uint32_t)1 <<  PIN_13);}
#define LD3_OFF     {GPIOC->BSRR = (uint32_t)((uint32_t)1 << (PIN_13 + 16));}
#define LD4_ON      {GPIOF->BSRR = (uint32_t)((uint32_t)1 <<  PIN_07);}
#define LD4_OFF     {GPIOF->BSRR = (uint32_t)((uint32_t)1 << (PIN_07 + 16));}
#define LD5_ON      {GPIOF->BSRR = (uint32_t)((uint32_t)1 <<  PIN_06);}
#define LD5_OFF     {GPIOF->BSRR = (uint32_t)((uint32_t)1 << (PIN_06 + 16));}
#define LD6_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_09);}
#define LD6_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_09 + 16));}

#define BPV1_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_01);}
#define BPV1_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_01 + 16));}
#define BPV2_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_00);}
#define BPV2_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_00 + 16));}
#define BPV3_ON      {GPIOB->BSRR = (uint32_t)((uint32_t)1 <<  PIN_06);}
#define BPV3_OFF     {GPIOB->BSRR = (uint32_t)((uint32_t)1 << (PIN_06 + 16));}
#define BPV4_ON      {GPIOB->BSRR = (uint32_t)((uint32_t)1 <<  PIN_07);}
#define BPV4_OFF     {GPIOB->BSRR = (uint32_t)((uint32_t)1 << (PIN_07 + 16));}
#define BPV5_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_08);}
#define BPV5_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_08 + 16));}
#define BPV6_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_14);}
#define BPV6_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_14 + 16));}

#define RED_ON      {GPIOC->BSRR = (uint32_t)((uint32_t)1 <<  PIN_15);}
#define RED_OFF     {GPIOC->BSRR = (uint32_t)((uint32_t)1 << (PIN_15 + 16));}
#define GREEN_ON      {GPIOC->BSRR = (uint32_t)((uint32_t)1 <<  PIN_14);}
#define GREEN_OFF     {GPIOC->BSRR = (uint32_t)((uint32_t)1 << (PIN_14 + 16));}


// Ext GPIO's
#define TEMP_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_10);}
#define TEMP_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_10 + 16));}

GPIO_InitTypeDef GPIO_InitStructure;

void clock_init(void);
void Init_Std_GPIOs(void);
void SYSCLKConfig_STOP(void);
void Init_Reset_GPIOs(void);
void Init_LPM_GPIOs(void);
void read_all_SDADC(uint16_t nrep); //16bit SDADC
void read_all_SDADC_noautorange(uint16_t nrep);
void SDADCreadall(void);
void SDADCCall(void);
void dmppt_wdt2200(void);//2200ms watchdog timer
void DMPPTLDval(void);
void DMPPTPVval(void);
void DMPPTRX2(void);
void I2Cdmppt(void);
void ADC_Configuration(void);
uint16_t readADC1(uint8_t channel);

void USART_Config(void);
void comp(uint64_t decval,uint8_t nr,uint8_t* dataout,uint16_t pos);
uint64_t dcomp(uint8_t nr,uint8_t* datain,uint16_t pos);
uint64_t pow(int base, int exp);

uint64_t TLDallR,uptime;
uint32_t sj,j,js,ATPV1,ATPV2,ATPV3,ATPV4,ATPV5,ATPV6,APVOUT1,APVOUT2,TPV1,TPV2,TPV3,TPV4,TPV5,TPV6,TPV1a,TPV2a,TPV3a,TPV4a,TPV5a,TPV6a,TPVall,TPVall2,TPVallA,TPVallP,PVOUT1,PVOUT2,PVOUT1a,PVOUT2a,i2ctime,Time,Time3 = 0;
uint32_t ADC16[8][10];// 0..6 channels 0=Value, 1=Gain, 2=Gain_calibration, 3=Real-value, 4=gain flag, 5=offset for 8x, 6=offset for 4x, 7=offset for 2x, 8=offset for 1x, 9=offset for 1/2x
uint32_t ADC16temp[3],tempADC1[15],EXTADC1[8],Vdmppt[22],Vdmppt20,DMPPTTPVmax,DMPPTTPVmax2,lastmpp,lastmpp1;
uint32_t DMPPTLD[65][6],DMPPTTPV[3][8];
uint16_t DMPPTV,exttemp1,realexttemp1,tempc1,exttemp2,realexttemp2,tempc2,inttemp,inttempcal,tcal1,tcal2,iyi,iyi1,duty1,txerror,rxerror1,rxerror2,TLDpower[7];
uint8_t dmppt_tx[55]={0},dmppt_tx2[55]={0},dmppt_rx0[12]={0},dmppt_rx[12]={0},dmppt_rx2[15]={0},dmppt_LD,ldmeasure,lastdmppt_LD,maxdmppt_LD,dmppt_TPV1,dmppt_TPV,dmppt_TPV_temp1,dmppt_TPV_temp2,ld235,ld146;
uint8_t txi,rxi,txcount,rxcount,tx2_busy,usart1s,tempH,tempL1,tempL2,LD[7],TPVs[7],react2,react3,redon1,k1,jjjj,iii,iii1,jjj,uverror,busyPV,enablePVOUT,noi2c,greentimer,dtemp[5]={0},busydelay,busydelay2,overtemp,overcurrentPV;




const uint32_t sdadc_gain[5] = {SDADC_Gain_1_2,SDADC_Gain_1,SDADC_Gain_2,SDADC_Gain_4,SDADC_Gain_8};
const float gain_real[5]= {0.5,1,2,4,8};
const uint16_t Therm[15]={2012,1985,1937,1868,1770,1633,1462,1264,1056,856,677,528,407,315,243};//thermistor 10K 3950
const float Therm2[15]={3.7037,2.0833,1.4493,1.0204,0.7299,0.5848,0.5051,0.4808,0.5,0.5587,0.6711,0.8264,1.0870,1.3889};
const uint16_t* TS_CAL1 = ((uint16_t*)0x1FFFF7B8);  //internal temp calibration for 30C
const uint16_t* TS_CAL2 = ((uint16_t*)0x1FFFF7C2);  //internal temp calibration for 110C


//=================================================================================================
//                                        Main routine.
//=================================================================================================

int main(void)
{

	SYSCLKConfig_STOP();
	clock_init();
	RCC_ClocksTypeDef rcc_clocks;

	RCC_GetClocksFreq(&rcc_clocks);
	SysTick_Config((rcc_clocks.SYSCLK_Frequency / 1000));


#if DMPPT==450

	EXTADC1[0]=0;
	EXTADC1[1]=1500;
	EXTADC1[2]=0;
	EXTADC1[3]=shunt1*10000000;
	EXTADC1[4]=0;
	EXTADC1[5]=shunt2*10000000;
	EXTADC1[6]=0;
	EXTADC1[7]=shunt3*10000000;

#endif

    delay_ms(1);
    Init_Reset_GPIOs();
    Init_LPM_GPIOs();
    Init_Std_GPIOs();
    USART_Config();
    init_all_SDADC();
    delay_ms(20);

    dmppt_wdt2200();

    SDADCCall();

	sDMPPT_Init();


	I2C_ITConfig(I2C2, I2C_IT_ADDRI | I2C_IT_RXI , ENABLE); //enable I2C interrupt on address and receive byte



	ADC_Configuration();
	TEMP_ON;
	duty1=2500;
	tempADC1[11]=0;
	dmppt_LD=0;
	lastdmppt_LD=0;
	ldmeasure=0;
	dmppt_TPV1=0;
	react2=0;
	react3=0;
	uverror=0;
	i2ctime=5000;
	busyPV=1;
	lastmpp=0;
	lastmpp1=0;
	overtemp=0;
	overcurrentPV=0;
	txcount=51;
	uptime=0;
	usart1s=0;
	dmppt_TPV=0;

	inttempcal= *TS_CAL1 - *TS_CAL2;
	tcal1=(*TS_CAL1)*1.079; // (3.3V/3.058V) so value for +30C
	tcal2=tcal1+((inttempcal/80)*70); //value for -40C



//=================================================================================================
//                                       Main loop
//=================================================================================================

  for (;;)
  {

	  Time3=0;
	  delay_ms(1);

	  read_all_SDADC_noautorange(6);//read raw 16bit ADC data
	  SDADCreadall(); // convert ADC data to current

	  greentimer++;
	  if (greentimer>8) {GREEN_OFF;}
	  if (greentimer>15 && redon1==0) {RED_ON;}

	  if (inttemp>=110) {if (overtemp<=100) {overtemp++;} } //110 means +70C (shifted by 40C)
	  else if (inttemp<=100) {if (overtemp>0) {overtemp--;} }

	       if (TPV1>=110000 || TPV2>=60000 || TPV3>=83000 || TPV4>=83000 || TPV5>=60000 || TPV6>=110000) {if (overcurrentPV<100) {overcurrentPV++;} }
	  else if (TPV1<97000 && TPV2<47000 && TPV3<72000 && TPV4<72000 && TPV5<47000 && TPV6<97000) {if (overcurrentPV!=0) {overcurrentPV--;} }


	  DMPPTRX2();  // i2c received data conversion

	  DMPPTLDval(); // calculate Load resistance

#if test1!=1
	  //----------------------------------------------------------------------------------------------------------------------------------------------------

	  if ((Vdmppt20<(vmppmin))||(busyPV==1))
	  {

		    if (react2==0 && dmppt_LD>=1) {if (lastdmppt_LD==16) {dmppt_LD=15;} else if (lastdmppt_LD==32) {dmppt_LD=31;} else {dmppt_LD=lastdmppt_LD;}react2=20;}
		    else{
			if      (dmppt_LD>=32 && LD[1]!=0)  {dmppt_LD = dmppt_LD - 32;}
			else if (dmppt_LD>=16 && LD[6]!=0)  {dmppt_LD = dmppt_LD - 16;}
			else if (dmppt_LD>=8  && LD[4]!=0)  {dmppt_LD = dmppt_LD - 8;}
			else if (dmppt_LD>=4  && LD[5]!=0)  {dmppt_LD = dmppt_LD - 4;}
			else if (dmppt_LD>=2  && LD[3]!=0)  {dmppt_LD = dmppt_LD - 2;}
			else    {dmppt_LD = 0;}
		        }
	  }


	  if (Vdmppt20>=vmppmax && uverror==0 && uptime>=10 && busyPV==0 && lastmpp>=1 && react2==0 && duty1==0)
	  {
		  lastdmppt_LD=dmppt_LD;

		  if (dmppt_LD<7) {dmppt_LD++;}
		  else if ((dmppt_LD>=7)  && (dmppt_LD<16)) {dmppt_LD++;}
		  else if ((dmppt_LD>=16) && (dmppt_LD<24)) {dmppt_LD=24;}
		  else if ((dmppt_LD>=24) && (dmppt_LD<32)) {dmppt_LD++;}
		  else if ((dmppt_LD>=32) && (dmppt_LD<56)) {dmppt_LD=56;}
		  else if ((dmppt_LD>=56) && (dmppt_LD<63)) {dmppt_LD++;}

		  if ((dmppt_LD==1)||(dmppt_LD==2)||(dmppt_LD==4)||(dmppt_LD==8)||(dmppt_LD==16)||(dmppt_LD==32)) {DMPPTLD[dmppt_LD][3]=0;ldmeasure=1;}
	  }

	  delay_ms(4);


	  if (((((Vdmppt[20]+Vdmppt[19])/(200/v30div)))>=(vmppmax+vmppmax)) && uverror==0 && uptime>=10 && busyPV==0 && lastmpp>=1 && react2==0 && ldmeasure==0 && duty1==0)
	  {
		  lastdmppt_LD=dmppt_LD;

		  if (dmppt_LD<7) {dmppt_LD++;}
		  else if ((dmppt_LD>=7)  && (dmppt_LD<16)) {dmppt_LD++;}
		  else if ((dmppt_LD>=16) && (dmppt_LD<24)) {dmppt_LD=24;}
		  else if ((dmppt_LD>=24) && (dmppt_LD<32)) {dmppt_LD++;}
		  else if ((dmppt_LD>=32) && (dmppt_LD<56)) {dmppt_LD=56;}
		  else if ((dmppt_LD>=56) && (dmppt_LD<63)) {dmppt_LD++;}

		  if ((dmppt_LD==1)||(dmppt_LD==2)||(dmppt_LD==4)||(dmppt_LD==8)||(dmppt_LD==16)||(dmppt_LD==32)) {DMPPTLD[dmppt_LD][3]=0;ldmeasure=1;}
	  }



      //------------------------------------------------------------------------------------------------------------------------------------------------------
#endif

#if test1==1

if (uptime>4) {LD1_ON;LD[1]=1;LD2_ON;LD[2]=1;LD3_ON;LD[3]=1;LD4_ON;LD[4]=1;LD5_ON;LD[5]=1;LD6_ON;LD[6]=1;}
if (uptime>8) {BPV1_ON;TPVs[1]=1;BPV2_ON;TPVs[2]=1;BPV3_ON;TPVs[3]=1;BPV4_ON;TPVs[4]=1;BPV5_ON;TPVs[5]=1;BPV6_ON;TPVs[6]=1;}
if (uptime<=4) {LD1_OFF;LD[1]=0;LD2_OFF;LD[2]=0;LD3_OFF;LD[3]=0;LD4_OFF;LD[4]=0;LD5_OFF;LD[5]=0;LD6_OFF;LD[6]=0; BPV1_OFF;TPVs[1]=0;BPV2_OFF;TPVs[2]=0;BPV3_OFF;TPVs[3]=0;BPV4_OFF;TPVs[4]=0;BPV5_OFF;TPVs[5]=0;BPV6_OFF;TPVs[6]=0;}
if (uptime>12) {uptime=0;}

#endif

      if (Time>=970)
      {
    	  IWDG_ReloadCounter(); //reload watchdog counter to 2.2s
    	  RED_OFF;
    	  if (i2ctime>2500){redon1=1;} //lost i2c communication for more than 2.5seconds
    	  if (react2>0) {react2--;}
    	  if (lastmpp<601){lastmpp++;}
    	  if (uptime<=432000) {uptime++;}
    	  iyi1++;
    	  DMPPTV=(((tempADC1[0]*v30div)/iyi));
    	  exttemp1=((tempADC1[1]/iyi));
    	  exttemp2=((tempADC1[2]/iyi));
    	  tempADC1[3]=(tempADC1[3]/iyi);
    	  if (tempADC1[3]<tcal2) {inttemp=((tcal2-tempADC1[3])/(inttempcal/80));}//((tempADC1[4]/iyi));
    	  else {inttemp=0;}
          ATPV1=TPV1a/iyi1; //needs testing
          ATPV2=TPV2a/iyi1;
          ATPV3=TPV3a/iyi1;
          ATPV4=TPV4a/iyi1;
          ATPV5=TPV5a/iyi1;
          ATPV6=TPV6a/iyi1;
          APVOUT1=PVOUT1a/iyi1;
          APVOUT2=PVOUT2a/iyi1;


          if (exttemp1>Therm[0]){realexttemp1=450;}//lower than -40C
          if (exttemp2>Therm[0]){realexttemp2=450;}//lower than -40C
          for (jjjj=0;jjjj<4;jjjj++){
          if ((exttemp1<=Therm[jjjj])&(exttemp1>Therm[jjjj+1])){realexttemp1=(300-(jjjj*100))+((exttemp1-Therm[jjjj+1])*Therm2[jjjj]);}
          if ((exttemp2<=Therm[jjjj])&(exttemp2>Therm[jjjj+1])){realexttemp2=(300-(jjjj*100))+((exttemp2-Therm[jjjj+1])*Therm2[jjjj]);}
          }
          for (jjjj=4;jjjj<14;jjjj++){
          if ((exttemp1<=Therm[jjjj])&(exttemp1>Therm[jjjj+1])){realexttemp1=((jjjj*100)-300)-((exttemp1-Therm[jjjj+1])*Therm2[jjjj]);}
          if ((exttemp2<=Therm[jjjj])&(exttemp2>Therm[jjjj+1])){realexttemp2=((jjjj*100)-300)-((exttemp2-Therm[jjjj+1])*Therm2[jjjj]);}
          }
          if (exttemp1<=Therm[14]){realexttemp1=999;}//higher than +99C
          if (exttemp2<=Therm[14]){realexttemp2=999;}//higher than +99C

      	  if (exttemp1<=Therm[4]){realexttemp1=realexttemp1+450;}
      	  else {realexttemp1=450-realexttemp1;}
      	  if (exttemp2<=Therm[4]){realexttemp2=realexttemp2+450;}
      	  else {realexttemp2=450-realexttemp2;}

      	  if (realexttemp1>=150) {tempc1=(realexttemp1-50)/10;}
      	  else {tempc1=0;}
      	  if (realexttemp2>=150) {tempc2=(realexttemp2-50)/10;}
      	  else {tempc2=0;}

        if (tempc1==0 || dmppt_rx2[7]==0) {ld235=0;}
        if (tempc1>=10 && dmppt_rx2[7]==1) {if ((tempc1+dtemp[0])>=dmppt_rx2[9]) {ld235=0;dtemp[0]=3;} else {ld235=1;dtemp[0]=0;} }
        if (tempc1>=10 && dmppt_rx2[7]==2) {if (tempc1<=(dmppt_rx2[9]+dtemp[1])) {ld235=0;dtemp[1]=3;} else {ld235=1;dtemp[1]=0;} }

        if (tempc2==0 || dmppt_rx2[8]==0) {ld146=0;}
        if (tempc2>=10 && dmppt_rx2[8]==1) {if ((tempc2+dtemp[2])>=dmppt_rx2[10]) {ld146=0;dtemp[2]=3;} else {ld146=1;dtemp[2]=0;} }
        if (tempc2>=10 && dmppt_rx2[8]==2) {if (tempc2<=(dmppt_rx2[10]+dtemp[3])) {ld146=0;dtemp[3]=3;} else {ld146=1;dtemp[3]=0;} }


          TPV1a=0;
          TPV2a=0;
          TPV3a=0;
          TPV4a=0;
          TPV5a=0;
          TPV6a=0;
          PVOUT1a=0;
          PVOUT2a=0;
    	  tempADC1[0]=0;
    	  tempADC1[1]=0;
    	  tempADC1[2]=0;
    	  tempADC1[3]=0;
    	  tempADC1[11]=iyi1;

          greentimer=0;
    	  Time=0;
    	  iyi=0;
    	  iyi1=0;
    	  usart1s=1;
    	  GREEN_ON;
    	  if (((DMPPTLD[1][3]!=0)||(DMPPTLD[2][3]!=0)||(DMPPTLD[4][3]!=0)||(DMPPTLD[8][3]!=0)||(DMPPTLD[16][3]!=0)||(DMPPTLD[32][3]!=0)) && (Vdmppt20>=vmppmax)) {react3++;}
    	  if (react3>=120) {dmppt_LD=0;react2=0;react3=0;}

      }
      else {iyi1++;}

      I2Cdmppt();

	  //if (dmppt_tx[43]>=110) {if (overtemp<100) {overtemp++;} }  //110 means +70C (shifted by 40C)
	  //else if (dmppt_tx[43]<=100) {if (overtemp>0) {overtemp--;} }

      // send dmppt data over USART maybe need to compress this in base 91
      if (usart1s==1)
      {
      for (sj=0;sj<51;sj++)
      {
	  while (USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
	  USART_SendData(USART3, dmppt_tx[sj]);
      }
  	  while (USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
  	  USART_SendData(USART3, 10);   //new line
	  usart1s=0;
      }


  }
}



void dmppt_wdt2200(void)
{

  IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
  IWDG_SetPrescaler(IWDG_Prescaler_32); //LSI is 40kHz so about 1.25kHz after prescaler
  IWDG_SetReload(2750); // counter almost at the top for about 2.2 seconds
  IWDG_ReloadCounter();
  IWDG_Enable();
}




void SDADCCall(void)
{

	for (j=0 ;j<8;++j)  {ADC16[j][1]=0;}

	read_all_SDADC_noautorange(11);

	    	for (js=0 ;js<5;++js){
	     	for (j=0 ;j<8;++j){
	     	ADC16[j][5+js]=0;  //set all ADC offset to 0
	     	     	}
	     	}

	    	for (j=0 ;j<8;++j)  {ADC16[j][1]=0;}  // set all channels to max gain of 1/2 (0)

	for (sj=0;sj<4;sj++)     // get the SDADC offset 4 average
	 {
		read_all_SDADC_noautorange(12);
		for (j=0;j<8;j++){ADC16[j][5]=ADC16[j][0]+ADC16[j][5];}
	 }

	for (j=0;j<8;j++){

	if (((ADC16[j][5]/4)<=163300)){ADC16[j][5]=163300;}
	else if (((ADC16[j][5]/4)>=164300)) {ADC16[j][5]=164300;}
	else { ADC16[j][5]=ADC16[j][5]/4; }

	}


}

void SDADCreadall(void)
{
	//ADC16 --  (7 - TPV4) (6 - TPV3) (5 - PVOUT2) (4 - TPV5) (3 - TPV6) (2 - TPV1) (1- TPV2) (0 - PVOUT1)

 if (ADC16[2][0]>ADC16[2][5+ADC16[2][1]]){TPV1=(((((((ADC16[2][0]-ADC16[2][5+ADC16[2][1]])/gain_real[ADC16[2][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[3]))*100);TPV1a=TPV1+TPV1a;}
 else {TPV1=0;TPV1a=TPV1+TPV1a;}
 if (ADC16[1][0]>ADC16[1][5+ADC16[1][1]]){TPV2=(((((((ADC16[1][0]-ADC16[1][5+ADC16[1][1]])/gain_real[ADC16[1][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[5]))*100);TPV2a=TPV2+TPV2a;}
 else {TPV2=0;TPV2a=TPV2+TPV2a;}
 if (ADC16[6][0]>ADC16[6][5+ADC16[6][1]]){TPV3=(((((((ADC16[6][0]-ADC16[6][5+ADC16[6][1]])/gain_real[ADC16[6][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[7]))*100);TPV3a=TPV3+TPV3a;}
 else {TPV3=0;TPV3a=TPV3+TPV3a;}
 if (ADC16[7][0]>ADC16[7][5+ADC16[7][1]]){TPV4=(((((((ADC16[7][0]-ADC16[7][5+ADC16[7][1]])/gain_real[ADC16[7][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[7]))*100);TPV4a=TPV4+TPV4a;}
 else {TPV4=0;TPV4a=TPV4+TPV4a;}
 if (ADC16[4][0]>ADC16[4][5+ADC16[4][1]]){TPV5=(((((((ADC16[4][0]-ADC16[4][5+ADC16[4][1]])/gain_real[ADC16[4][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[5]))*100);TPV5a=TPV5+TPV5a;}
 else {TPV5=0;TPV5a=TPV5+TPV5a;}
 if (ADC16[3][0]>ADC16[3][5+ADC16[3][1]]){TPV6=(((((((ADC16[3][0]-ADC16[3][5+ADC16[3][1]])/gain_real[ADC16[3][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[3]))*100);TPV6a=TPV6+TPV6a;}
 else {TPV6=0;TPV6a=TPV6+TPV6a;}

 if (ADC16[0][0]>ADC16[0][5+ADC16[0][1]]){PVOUT1=(((((((ADC16[0][0]-ADC16[0][5+ADC16[0][1]])/gain_real[ADC16[0][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[5]))*100);PVOUT1a=PVOUT1+PVOUT1a;}
 else {PVOUT1=0;PVOUT1a=PVOUT1+PVOUT1a;}
 if (ADC16[5][0]>ADC16[5][5+ADC16[5][1]]){PVOUT2=(((((((ADC16[5][0]-ADC16[5][5+ADC16[5][1]])/gain_real[ADC16[5][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[5]))*100);PVOUT2a=PVOUT2+PVOUT2a;}
 else {PVOUT2=0;PVOUT2a=PVOUT2+PVOUT2a;}

 Vdmppt20=0;
 for (iii1=0;iii1<20;iii1++) {Vdmppt20=Vdmppt[iii1]+Vdmppt20;}
 Vdmppt20 = Vdmppt20/(2000/v30div);

 TPVall=(TPV1+TPV2+TPV3+TPV4+TPV5+TPV6);
 TPVall2=TPVall+PVOUT1+PVOUT2;
 TPVallA=TPVall/100;

 TLDallR=( (10000000/(DMPPTLD[1][4]*LD[2])) + (10000000/(DMPPTLD[2][4]*LD[3])) + (10000000/(DMPPTLD[4][4]*LD[5])) + (10000000/(DMPPTLD[8][4]*LD[4])) + (10000000/(DMPPTLD[16][4]*LD[6])) + (10000000/(DMPPTLD[32][4]*LD[1])) );

 TPVallP=((TPV1+TPV2+TPV3+TPV4+TPV5+TPV6)*Vdmppt20)/1000;

}

void DMPPTRX2(void)
{
if (rxcount==10) { for (rxi=0;rxi<=9;rxi++) {dmppt_rx[rxi]=dmppt_rx0[rxi];}; }

rxerror1=((dmppt_rx[8]*256)+dmppt_rx[9]);
rxerror2=(dmppt_rx[0]+dmppt_rx[1]+dmppt_rx[2]+dmppt_rx[3]+dmppt_rx[4]+dmppt_rx[5]+dmppt_rx[6]+dmppt_rx[7]);
if (i2ctime<=3000 && (dmppt_rx[0]>=dmppt_rx[1]) && (dmppt_rx[6]<=140) && (dmppt_rx[7]<=140) && (dmppt_rx[0]==0 || dmppt_rx[0]==40 || dmppt_rx[0]==60 || dmppt_rx[0]==100 || dmppt_rx[0]==120) && rxerror1==rxerror2) {dmppt_rx2[12]=1;dmppt_rx2[0]=1;}
else
{
if (i2ctime>3500)
{
dmppt_rx2[0]=0;
dmppt_rx2[1]=0;
dmppt_rx2[2]=0;
dmppt_rx2[3]=0;
dmppt_rx2[4]=0;
dmppt_rx2[5]=0;
dmppt_rx2[6]=0;
dmppt_rx2[7]=1;   //default to heating for TLD235 in case of lost communication with SBMS
dmppt_rx2[8]=1;   //default to heating for TLD146 in case of lost communication with SBMS
dmppt_rx2[9]=100; //fixed +60C limit for TLD235 in case of lost communication with SBMS
dmppt_rx2[10]=100;//fixed +60C limit for TLD146 in case of lost communication with SBMS

}
dmppt_rx2[12]=0;
}


if (dmppt_rx2[0]==1 && dmppt_rx2[12]==1)
{
	if (dmppt_rx[1]<=112){dmppt_rx2[1]=dmppt_rx[1];}//max charge current
	else if (dmppt_rx[1]>112) {dmppt_rx2[1]=112;}

	dmppt_rx2[2]=dmppt_rx[2];//PV1 current
	dmppt_rx2[3]=dmppt_rx[3];//PV2 current
	dmppt_rx2[4]=dmppt_rx[4];//Batt current
	tempL1=(dmppt_rx[5] & 128);
if (tempL1>0) {dmppt_rx2[5]=1;} else {dmppt_rx2[5]=0;} //PV1
    tempL1=(dmppt_rx[5] & 64);
if (tempL1>0) {dmppt_rx2[6]=1;} else {dmppt_rx2[6]=0;} //PV2

    tempL1=(dmppt_rx[5] & 32);
    tempL2=(dmppt_rx[5] & 16);
if ((tempL1>0 && tempL2>0)||(tempL1==0 && tempL2==0)) {dmppt_rx2[7]=0;}//TLD235 OFF
else if (tempL1>0 && tempL2==0) {dmppt_rx2[7]=1;}//TLD235 Heat
else if (tempL1==0 && tempL2>0) {dmppt_rx2[7]=2;}//TLD235 Cool

    tempL1=(dmppt_rx[5] & 8);
    tempL2=(dmppt_rx[5] & 4);
if ((tempL1>0 && tempL2>0)||(tempL1==0 && tempL2==0)) {dmppt_rx2[8]=0;}//TLD146 OFF
else if (tempL1>0 && tempL2==0) {dmppt_rx2[8]=1;}//TLD146 Heat
else if (tempL1==0 && tempL2>0) {dmppt_rx2[8]=2;}//TLD146 Cool


    dmppt_rx2[9] =dmppt_rx[6]; //EXT1 Limit
    dmppt_rx2[10]=dmppt_rx[7]; //EXT2 Limit

dmppt_rx2[12]=0;

}

if (dmppt_rx2[0]==1) {DMPPTPVval();noi2c=0;}
else {   if (noi2c<22) {noi2c++;} else {dmppt_TPV=0;busyPV=0;lastmpp=570;}  }

}



void DMPPTLDval(void)  //calculate LD resistance
{

if ((duty1==0) && (ldmeasure!=0) && (Vdmppt20>=vmppmin) && ((dmppt_LD==1)||(dmppt_LD==2)||(dmppt_LD==4)||(dmppt_LD==8)||(dmppt_LD==16)||(dmppt_LD==32)))
{
	  ldmeasure=0;
	  read_all_SDADC_noautorange(6);//read raw 16bit ADC data
	  SDADCreadall(); // convert ADC data to current


	  if (dmppt_LD==1)
	  {
		  DMPPTLD[dmppt_LD][0]=Vdmppt20;
		  DMPPTLD[dmppt_LD][1]=TPVallA;
		  //DMPPTLD[dmppt_LD][2]=TPVallP;
		  if      (TPVallA>=amin)  {DMPPTLD[dmppt_LD][3]=0;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else if (TPVallA>=amax50){DMPPTLD[dmppt_LD][3]=2;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else                     {DMPPTLD[dmppt_LD][3]=1;DMPPTLD[dmppt_LD][4]=0;}
	  }
	  if (dmppt_LD==2)
	  {
		  DMPPTLD[dmppt_LD][0]=Vdmppt20;
		  DMPPTLD[dmppt_LD][1]=TPVallA;
		  //DMPPTLD[dmppt_LD][2]=TPVallP;
		  if      (TPVallA>=amin)  {DMPPTLD[dmppt_LD][3]=0;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else if (TPVallA>=amax50){DMPPTLD[dmppt_LD][3]=2;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else                     {DMPPTLD[dmppt_LD][3]=1;DMPPTLD[dmppt_LD][4]=0;}
	  }
	  if (dmppt_LD==4)
	  {
		  DMPPTLD[dmppt_LD][0]=Vdmppt20;
		  DMPPTLD[dmppt_LD][1]=TPVallA;
		  //DMPPTLD[dmppt_LD][2]=TPVallP;
		  if      (TPVallA>=amin)  {DMPPTLD[dmppt_LD][3]=0;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else if (TPVallA>=amax50){DMPPTLD[dmppt_LD][3]=2;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else                     {DMPPTLD[dmppt_LD][3]=1;DMPPTLD[dmppt_LD][4]=0;}
	  }
	  if (dmppt_LD==8)
	  {
		  DMPPTLD[dmppt_LD][0]=Vdmppt20;
		  DMPPTLD[dmppt_LD][1]=TPVallA;
		  //DMPPTLD[dmppt_LD][2]=TPVallP;
		  if      (TPVallA>=amin)   {DMPPTLD[dmppt_LD][3]=0;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else if (TPVallA>=amax100){DMPPTLD[dmppt_LD][3]=2;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else                      {DMPPTLD[dmppt_LD][3]=1;DMPPTLD[dmppt_LD][4]=0;}

	  }
	  if (dmppt_LD==16)
	  {
		  DMPPTLD[dmppt_LD][0]=Vdmppt20;
		  DMPPTLD[dmppt_LD][1]=TPVallA;
		  //DMPPTLD[dmppt_LD][2]=TPVallP;
		  if      (TPVallA>=amin)   {DMPPTLD[dmppt_LD][3]=0;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else if (TPVallA>=amax100){DMPPTLD[dmppt_LD][3]=2;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else                      {DMPPTLD[dmppt_LD][3]=1;DMPPTLD[dmppt_LD][4]=0;}
	  }
	  if (dmppt_LD==32)
	  {
		  DMPPTLD[dmppt_LD][0]=Vdmppt20;
		  DMPPTLD[dmppt_LD][1]=TPVallA;
		  //DMPPTLD[dmppt_LD][2]=TPVallP;
		  if      (TPVallA>=amin)   {DMPPTLD[dmppt_LD][3]=0;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else if (TPVallA>=amax100){DMPPTLD[dmppt_LD][3]=2;DMPPTLD[dmppt_LD][4]=(Vdmppt20*10000)/TPVallA;}
		  else                      {DMPPTLD[dmppt_LD][3]=1;DMPPTLD[dmppt_LD][4]=0;}
	  }



}

}

void DMPPTPVval(void)
{


// 3---------------------------------------------------------------------------------------
if (busyPV>=3) {busyPV=0;lastmpp=0;lastmpp1=0;react2=0;}
// ----------------------------------------------------------------------------------------

// 2---------------------------------------------------------------------------------------

if ((busyPV==2) && (busydelay==0)) {dmppt_TPV=0;}

if (busyPV==2 && busydelay<=4) { if (busydelay!=0) {dmppt_TPV=dmppt_TPV1;} busydelay++;}

if (busyPV==2 && busydelay>=4)
{
busydelay=0;


if (enablePVOUT==1)
{

DMPPTTPV[1][3]=DMPPTTPV[1][1]+DMPPTTPV[1][2];
DMPPTTPV[1][5]=DMPPTTPV[1][4]+DMPPTTPV[1][1];
DMPPTTPV[1][6]=DMPPTTPV[1][4]+DMPPTTPV[1][2];
DMPPTTPV[1][7]=DMPPTTPV[1][4]+DMPPTTPV[1][2]+DMPPTTPV[1][1];
DMPPTTPV[2][3]=DMPPTTPV[2][1]+DMPPTTPV[2][2];
DMPPTTPV[2][5]=DMPPTTPV[2][4]+DMPPTTPV[2][1];
DMPPTTPV[2][6]=DMPPTTPV[2][4]+DMPPTTPV[2][2];
DMPPTTPV[2][7]=DMPPTTPV[2][4]+DMPPTTPV[2][2]+DMPPTTPV[2][1];

if (dmppt_rx2[6]==0)
{
	DMPPTTPV[2][1]=0;
	DMPPTTPV[2][2]=0;
	DMPPTTPV[2][3]=0;
	DMPPTTPV[2][4]=0;
	DMPPTTPV[2][5]=0;
	DMPPTTPV[2][6]=0;
	DMPPTTPV[2][7]=0;
}


DMPPTTPV[1][0]=0;
DMPPTTPV[2][0]=0;

DMPPTTPVmax=0;
dmppt_TPV_temp1=0;
dmppt_TPV_temp2=0;

if (dmppt_rx2[1]>9){
for (iii=0;iii<8;iii++)
{
for (jjj=0;jjj<8;jjj++)
{
if ((((DMPPTTPV[1][iii] + DMPPTTPV[2][jjj])+3000) <= ((dmppt_rx2[1]*900))) && ((DMPPTTPV[1][iii]+2000) < 56000) && ((DMPPTTPV[2][jjj]+2000) < 56000) && (DMPPTTPVmax <= (DMPPTTPV[1][iii] + DMPPTTPV[2][jjj])) )
{
dmppt_TPV_temp1=iii;
dmppt_TPV_temp2=jjj;
DMPPTTPVmax = DMPPTTPV[1][dmppt_TPV_temp1] + DMPPTTPV[2][dmppt_TPV_temp2];
}
}
}
}
else {dmppt_TPV_temp2=0;dmppt_TPV_temp1=0;}
if (dmppt_rx2[6]!=1){dmppt_TPV_temp2=0;}
if (dmppt_rx2[5]!=1){dmppt_TPV_temp1=0;}

dmppt_TPV=dmppt_TPV_temp1 + (dmppt_TPV_temp2*8);
busyPV=3;
busydelay2=0;
}

if      (dmppt_TPV==1) {DMPPTTPV[1][1]=PVOUT1+TPV2;}
else if (dmppt_TPV==2) {DMPPTTPV[1][2]=PVOUT1+TPV3;}
else if (dmppt_TPV==4) {DMPPTTPV[1][4]=PVOUT1+TPV1;}
else if (dmppt_TPV==8) {DMPPTTPV[2][1]=PVOUT2+TPV5;}
else if (dmppt_TPV==16){DMPPTTPV[2][2]=PVOUT2+TPV4;}
else if (dmppt_TPV==32){DMPPTTPV[2][4]=PVOUT2+TPV6;dmppt_TPV=0;enablePVOUT=1;}

if (dmppt_TPV1<=16 && busyPV==2 && enablePVOUT!=1) { dmppt_TPV1=dmppt_TPV1+dmppt_TPV1; }
}




//-------------------------------------------------------------------------------------------

// 1-----------------------------------------------------------------------------------------
if (busyPV==1) {dmppt_TPV1=1;dmppt_TPV=0;busydelay=0;enablePVOUT=0;}
if (busyPV==1 && dmppt_LD==0) {busyPV=2;react3=120;}
// ------------------------------------------------------------------------------------------

// 0-----------------------------------------------------------------------------------------
if (busyPV==0)
{


	if (dmppt_rx2[6]==1 && dmppt_rx2[5]==1)
	{
	if ( ((PVOUT1 + PVOUT2) > ((dmppt_rx2[1]*1000)+2000)) || (PVOUT1 > 62000) || (PVOUT2 > 62000) ){if (busydelay2>=4) {busyPV=1;dmppt_TPV=0;} else {busydelay2++;} }
	else{busydelay2=0;}
	if ( ((PVOUT1 + PVOUT2) > ((dmppt_rx2[1]*1000)+8000)) || (PVOUT1 > 64000) || (PVOUT2 > 64000) ){busyPV=1;dmppt_TPV=0;}

	if ( (((PVOUT1 + PVOUT2)*1.33) < (DMPPTTPVmax)) && dmppt_TPV<63 && lastmpp>=10) {if (lastmpp1<70) {lastmpp1++;}}
	else {lastmpp1=0;}

	if (dmppt_rx2[1]<=9 || dmppt_rx2[1]>120){busyPV=0;dmppt_TPV=0;busydelay2=0;}

	}

	else if (dmppt_rx2[6]==0 && dmppt_rx2[5]==1)
	{
	if ( ((PVOUT1) > ((dmppt_rx2[1]*1000)+2000)) || (PVOUT1 > 42000)){if (busydelay2>=4) {busyPV=1;dmppt_TPV=0;} else {busydelay2++;}}
	else{busydelay2=0;}
	if ( ((PVOUT1) > ((dmppt_rx2[1]*1000)+4000)) || (PVOUT1 > 43000)){busyPV=1;dmppt_TPV=0;}

	if ( (((PVOUT1)*1.33) < (DMPPTTPVmax)) && dmppt_TPV<7 && lastmpp>=10) {if (lastmpp1<70) {lastmpp1++;}}
	else {lastmpp1=0;}

	if (dmppt_rx2[1]<=9 || dmppt_rx2[1]>120){busyPV=0;dmppt_TPV=0;busydelay2=0;}
	}

	else {dmppt_TPV=0;}

	if (lastmpp1>=65) {busyPV=1;}
	if (lastmpp>=600) {busyPV=1;}

}
// -----------------------------------------------------------------------------------------

}


void I2Cdmppt(void) //buffer data to be sent to SBMS
{

if (txcount>=51)
{
	dmppt_tx[0] = softver;
	dmppt_tx[1] = model;

    dmppt_tx[2] = (DMPPTV & 0x0000ff00) >> 8;
    dmppt_tx[3] = (DMPPTV & 0x000000ff) ;

    dmppt_tx[4] = ((TPVs[1]*1)+(TPVs[2]*2)+(TPVs[3]*4)+(TPVs[4]*8)+(TPVs[5]*16)+(TPVs[6]*32));//dmppt_TPV;


    dmppt_tx[5] = (ATPV1 & 0x00030000) >> 16;
    dmppt_tx[6] = (ATPV1 & 0x0000ff00) >> 8;
    dmppt_tx[7] = (ATPV1 & 0x000000ff) ;

    dmppt_tx[8] = (ATPV2 & 0x00030000) >> 16;
    dmppt_tx[9] = (ATPV2 & 0x0000ff00) >> 8;
    dmppt_tx[10] = (ATPV2 & 0x000000ff) ;

    dmppt_tx[11] = (ATPV3 & 0x00030000) >> 16;
    dmppt_tx[12] = (ATPV3 & 0x0000ff00) >> 8;
    dmppt_tx[13] = (ATPV3 & 0x000000ff) ;

    dmppt_tx[14] = (ATPV4 & 0x00030000) >> 16;
    dmppt_tx[15] = (ATPV4 & 0x0000ff00) >> 8;
    dmppt_tx[16] = (ATPV4 & 0x000000ff) ;

    dmppt_tx[17] = (ATPV5 & 0x00030000) >> 16;
    dmppt_tx[18] = (ATPV5 & 0x0000ff00) >> 8;
    dmppt_tx[19] = (ATPV5 & 0x000000ff) ;

    dmppt_tx[20] = (ATPV6 & 0x00030000) >> 16;
    dmppt_tx[21] = (ATPV6 & 0x0000ff00) >> 8;
    dmppt_tx[22] = (ATPV6 & 0x000000ff) ;

    dmppt_tx[23] = (APVOUT1 & 0x00030000) >> 16;
    dmppt_tx[24] = (APVOUT1 & 0x0000ff00) >> 8;
    dmppt_tx[25] = (APVOUT1 & 0x000000ff) ;

    dmppt_tx[26] = (APVOUT2 & 0x00030000) >> 16;
    dmppt_tx[27] = (APVOUT2 & 0x0000ff00) >> 8;
    dmppt_tx[28] = (APVOUT2 & 0x000000ff) ;


    dmppt_tx[29]=(LD[1]*1) + (LD[2]*2) + (LD[3]*4) + (LD[4]*8) + (LD[5]*16) + (LD[6]*32);


//---------

    if (LD[2]!=0 && TLDallR!=0 && DMPPTLD[1][4]!=0)
    {
    TLDpower[1]=((10000000/DMPPTLD[1][4])*TPVallP)/TLDallR;
    dmppt_tx[32] = TLDpower[1] >> 8 ;
    dmppt_tx[33] = TLDpower[1];
    }
    else {dmppt_tx[32] = 0; dmppt_tx[33] = 0;}

//---------
    if (LD[3]!=0 && TLDallR!=0 && DMPPTLD[2][4]!=0)
    {
    TLDpower[2]=((10000000/DMPPTLD[2][4])*TPVallP)/TLDallR;
    dmppt_tx[34] = TLDpower[2] >> 8 ;
    dmppt_tx[35] = TLDpower[2];
    }
    else {dmppt_tx[34] = 0; dmppt_tx[35] = 0;}
//---------
    if (LD[5]!=0 && TLDallR!=0 && DMPPTLD[4][4]!=0)
    {
    TLDpower[3]=((10000000/DMPPTLD[4][4])*TPVallP)/TLDallR;
    dmppt_tx[38] = TLDpower[3] >> 8 ;
    dmppt_tx[39] = TLDpower[3];
    }
    else {dmppt_tx[38] = 0; dmppt_tx[39] = 0;}
//---------
    if (LD[4]!=0 && TLDallR!=0 && DMPPTLD[8][4]!=0)
    {
    TLDpower[4]=((10000000/DMPPTLD[8][4])*TPVallP)/TLDallR;
    dmppt_tx[36] = TLDpower[4] >> 8 ;
    dmppt_tx[37] = TLDpower[4];
    }
    else {dmppt_tx[36] = 0; dmppt_tx[37] = 0;}
//---------
    if (LD[6]!=0 && TLDallR!=0 && DMPPTLD[16][4]!=0)
    {
    TLDpower[5]=((10000000/DMPPTLD[16][4])*TPVallP)/TLDallR;
    dmppt_tx[40] = TLDpower[5] >> 8 ;
    dmppt_tx[41] = TLDpower[5];
    }
    else {dmppt_tx[40] = 0; dmppt_tx[41] = 0;}
//---------
    if (LD[1]!=0 && TLDallR!=0 && DMPPTLD[32][4]!=0)
    {
    TLDpower[6]=((10000000/DMPPTLD[32][4])*TPVallP)/TLDallR;
    dmppt_tx[30] = TLDpower[6] >> 8 ;
    dmppt_tx[31] = TLDpower[6];
    }
    else {dmppt_tx[30] = 0; dmppt_tx[31] = 0;}
//---------
    dmppt_tx[42] = (inttemp & 0x000000ff) ;

    dmppt_tx[43] = (realexttemp1 & 0x0000ff00) >> 8 ;
    dmppt_tx[44] = (realexttemp1 & 0x000000ff) ;

    dmppt_tx[45] = (realexttemp2 & 0x0000ff00) >> 8 ;
    dmppt_tx[46] = (realexttemp2 & 0x000000ff) ;

    DMPPTTPVmax2=DMPPTTPVmax/10;
    dmppt_tx[47] = (DMPPTTPVmax2 & 0xff00) >> 8;
    dmppt_tx[48] = (DMPPTTPVmax2 & 0x00ff);
     //tempADC1[11]; //number of loops in a second
    txerror=0;
    for (txi=0;txi<=48;txi++){txerror=dmppt_tx[txi]+txerror;}//use all 48 data for error correction.

    dmppt_tx[49] = (txerror & 0xff00) >> 8 ;
    dmppt_tx[50] = (txerror & 0x00ff);
}

    if (txcount>=51){tx2_busy=1; for (txi=0;txi<=50;txi++) {dmppt_tx2[txi]=dmppt_tx[txi];}; tx2_busy=0;}


}



void I2C2_EV_IRQHandler(void)
{

	if (I2C_GetITStatus(I2C2, I2C_IT_ADDR) != RESET)  // DMPPT address received
	{
		   if (I2C_GetTransferDirection(I2C2)==I2C_Direction_Transmitter){rxcount=0;I2C_ITConfig(I2C2, I2C_IT_TXI, DISABLE);i2ctime=0;redon1=0;}
		   else if (I2C_GetTransferDirection(I2C2)==I2C_Direction_Receiver) {txcount=1;I2C_SendData(I2C2,dmppt_tx[0]);I2C_ITConfig(I2C2, I2C_IT_TXI, ENABLE);}
		   else {}
	       I2C_ClearITPendingBit(I2C2, I2C_IT_ADDR);

	}

	else if (I2C_GetITStatus(I2C2, I2C_IT_RXNE))  // read data from SBMS
	{
		   dmppt_rx2[12]=0;
	       dmppt_rx0[rxcount++]= I2C2 ->RXDR;
	}

	else if (I2C_GetITStatus(I2C2, I2C_IT_TXIS))  // send data to SBMS
	{
		   if ((txcount<51) && (tx2_busy=1)){I2C_SendData(I2C2,dmppt_tx[txcount]);txcount++;}
		   else if ((txcount<51) && (tx2_busy=0)){I2C_SendData(I2C2,dmppt_tx2[txcount]);txcount++;}
		   else {I2C_ClearITPendingBit(I2C2, I2C_IT_TXIS);I2C_ITConfig(I2C2, I2C_IT_TXI, DISABLE);}
	}
	else {I2C_ITConfig(I2C2, I2C_IT_TXI, DISABLE);}

	I2C_ClearFlag(sDMPPT_I2C, I2C_ICR_NACKCF);
}


u16 readADC1(u8 channel) //12bit ADC read
{
  ADC_RegularChannelConfig(ADC1, channel, 1, ADC_SampleTime_7Cycles5);
  // Start the conversion
  ADC_SoftwareStartConv(ADC1);
  // Wait until conversion completion
  while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
  // Get the conversion value
  return ADC_GetConversionValue(ADC1);
}


void ADC_Configuration(void) //12bit ADC configuration
{
  //ADC_CommonInitTypeDef CommonInitStruct;
  ADC_InitTypeDef  ADC_InitStructure;
  /* PCLK2 is the APB2 clock */
  /* ADCCLK = PCLK2/8 = 40/8 = 5MHz*/
  RCC_ADCCLKConfig(RCC_PCLK2_Div8);

  /* Enable ADC1 clock so that we can talk to it */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
  /* Put everything back to power-on defaults */
  ADC_DeInit(ADC1);

  /* ADC1 Configuration ------------------------------------------------------*/
  //ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
  /* Disable the scan conversion so we do one at a time */
  ADC_InitStructure.ADC_ScanConvMode = DISABLE;
  /* Don't do contimuous conversions - do them on demand */
  ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
  /* Start conversin by software, not an external trigger */
  ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
  /* Conversions are 12 bit - put them in the lower 12 bits of the result */
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  /* Say how many channels would be used by the sequencer */
  ADC_InitStructure.ADC_NbrOfChannel = 1;

  /* Now do the setup */
  ADC_Init(ADC1, &ADC_InitStructure);
  /* Enable ADC1 */
  ADC_Cmd(ADC1, ENABLE);

  /* Enable ADC1 reset calibaration register */
  ADC_ResetCalibration(ADC1);
  /* Check the end of ADC1 reset calibration register */
  while(ADC_GetResetCalibrationStatus(ADC1));
  /* Start ADC1 calibaration */
  ADC_StartCalibration(ADC1);
  /* Check the end of ADC1 calibration */
  while(ADC_GetCalibrationStatus(ADC1));
}


//=================================================================================================
//      Initializes GPIOs
//=================================================================================================

void Init_Std_GPIOs(void)
{

	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);


	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_Init(GPIOC, &GPIO_InitStructure);

	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOF, ENABLE);


	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_Init(GPIOF, &GPIO_InitStructure);


	 RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);


	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_Init(GPIOB, &GPIO_InitStructure);

	  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_9 | GPIO_Pin_1 | GPIO_Pin_0 | GPIO_Pin_8 | GPIO_Pin_14;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_Init(GPIOA, &GPIO_InitStructure);

	  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);


	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_Init(GPIOA, &GPIO_InitStructure);


	  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	  /* ADC1 */
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_5 | GPIO_Pin_4;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_Init(GPIOA, &GPIO_InitStructure);

}

//=====================================================================================

void comp(uint64_t decval,uint8_t nr,uint8_t* dataout,uint16_t pos)
{
uint8_t i;
uint64_t qw[8],qe[8];

for (i=0;i<nr;i++){qw[i]=qe[i]=0;}
qe[0] = decval;
for (i=0;i<nr;i++){	if(qe[i]>90){qe[i+1] =(qe[i]/91);qw[i] =(qe[i]-(qe[i+1]*91));}else {qw[i]=qe[i];}}
for (i=0;i<nr;i++){dataout[pos+i]=qw[nr-(i+1)]+35;}

}

//function dcmp(p,s,d){xx=0; for (z=0;z<s;z++){xx = xx + ((d.charCodeAt((p+s-1)-z)-35)*Math.pow(91,z));}return xx;}

//=====================================================================================

uint64_t dcomp(uint8_t nr,uint8_t* datain,uint16_t pos)
{
uint8_t i;
uint64_t decval;

decval=0;
for (i=0;i<nr;i++)
{
decval=decval+((datain[(pos+nr-1)-i]-35)*pow(91,i));
}
return decval;
}

//=================================================================================================

//=================================================================================================
//                                  Configures the USART.
//=================================================================================================

void USART_Config(void)
{
	  USART_InitTypeDef USART_InitStructure;
	  GPIO_InitTypeDef GPIO_InitStructure;
	  //NVIC_InitTypeDef NVIC_InitStructure;

	  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	  RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3,ENABLE);

	  GPIO_PinAFConfig(GPIOB, GPIO_PinSource8, GPIO_AF_7);
	  GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_7);

	  /* Configure USART3 pins:  Rx  ----------------------------*/
	  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_9;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	  GPIO_Init(GPIOB, &GPIO_InitStructure);

	  /* Configure USART3 pins:  Tx  ----------------------------*/
	  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_8;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	  GPIO_Init(GPIOB, &GPIO_InitStructure);


	  /* Enable USART3 IRQ */
	  //NVIC_InitStructure.NVIC_IRQChannel = USART3_IRQn;
	  //NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	  //NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	  //NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	  //NVIC_Init(&NVIC_InitStructure);


	  USART_InitStructure.USART_BaudRate = (115200);
	  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	  USART_InitStructure.USART_StopBits = USART_StopBits_1;
	  USART_InitStructure.USART_Parity = USART_Parity_No;
	  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	  USART_Init(USART3, &USART_InitStructure);

	  //USART_ITConfig(USART3, USART_IT_RXNE, ENABLE); // Receive interrupt ...

	  USART_Cmd(USART3,ENABLE);

}


//=================================================================================================
//                         SDADC read values all no autorange
//=================================================================================================
void read_all_SDADC_noautorange(uint16_t nrep)
{

	uint8_t nrep1;


    nrep1=nrep;
    nrep=nrep*5;


	init_SDADC1(SDADC_Channel_4,sdadc_gain[ADC16[0][1]]); //SDADC_Gain_1_2,SDADC_Gain_1,SDADC_Gain_2,SDADC_Gain_4,SDADC_Gain_8
	init_SDADC3(SDADC_Channel_6,sdadc_gain[ADC16[3][1]]);
	init_SDADC2(SDADC_Channel_8,sdadc_gain[ADC16[6][1]]);

	read_all_SDADC(nrep);

    ADC16[0][0]=ADC16temp[0]/nrep1;
	ADC16[3][0]=ADC16temp[1]/nrep1;
	ADC16[6][0]=ADC16temp[2]/nrep1;
	//tempadc6=ADC16temp[2]/nrep1;


	init_SDADC1(SDADC_Channel_5,sdadc_gain[ADC16[1][1]]);
	init_SDADC3(SDADC_Channel_7,sdadc_gain[ADC16[4][1]]);
	init_SDADC2(SDADC_Channel_7,sdadc_gain[ADC16[7][1]]);

	read_all_SDADC(nrep);

	ADC16[1][0]=ADC16temp[0]/nrep1;
	ADC16[4][0]=ADC16temp[1]/nrep1;
	ADC16[7][0]=ADC16temp[2]/nrep1;
	//tempadc6=tempadc6+(ADC16temp[2]/nrep1);

	init_SDADC1(SDADC_Channel_6,sdadc_gain[ADC16[2][1]]);
	init_SDADC3(SDADC_Channel_8,sdadc_gain[ADC16[5][1]]);
	//init_SDADC2(SDADC_Channel_8,sdadc_gain[ADC16[6][1]]);

	read_all_SDADC(nrep);
	ADC16[2][0]=ADC16temp[0]/nrep1;
	ADC16[5][0]=ADC16temp[1]/nrep1;
	//ADC16[6][0]=(tempadc6+(ADC16temp[2]/nrep1))/3;


}


//=================================================================================================
//                                SDADC read values.
//=================================================================================================
void read_all_SDADC(uint16_t nrep)
{

ADC16temp[0]=0;
ADC16temp[1]=0;
ADC16temp[2]=0;
//discard first one


SDADC_SoftwareStartConv(SDADC1);
SDADC_SoftwareStartConv(SDADC2);
SDADC_SoftwareStartConv(SDADC3);

while(SDADC_GetFlagStatus(SDADC1,SDADC_FLAG_REOC)!=SET);
SDADC_GetConversionValue(SDADC1);
while(SDADC_GetFlagStatus(SDADC2,SDADC_FLAG_REOC)!=SET);
SDADC_GetConversionValue(SDADC2);
while(SDADC_GetFlagStatus(SDADC3,SDADC_FLAG_REOC)!=SET);
SDADC_GetConversionValue(SDADC3);


//uint32_t r=0;
do
	{
	SDADC_SoftwareStartConv(SDADC1);
	SDADC_SoftwareStartConv(SDADC2);
	SDADC_SoftwareStartConv(SDADC3);

	while(SDADC_GetFlagStatus(SDADC1,SDADC_FLAG_REOC)!=SET);
	ADC16temp[0] += (uint16_t) ((uint16_t)32768+SDADC_GetConversionValue(SDADC1));
	while(SDADC_GetFlagStatus(SDADC2,SDADC_FLAG_REOC)!=SET);
	ADC16temp[2] += (uint16_t) ((uint16_t)32768+SDADC_GetConversionValue(SDADC2));
	while(SDADC_GetFlagStatus(SDADC3,SDADC_FLAG_REOC)!=SET);
	ADC16temp[1] += (uint16_t) ((uint16_t)32768+SDADC_GetConversionValue(SDADC3));

	}
while(--nrep);


}


uint64_t pow(int base, int exp)
    {   int result = 1;
        while (exp)
        {
            if (exp & 1)
            result *= base;
            exp >>= 1;
            base *= base;
        }
        return result;
    }




//=================================================================================================
//      Configures system clock after wake-up from STOP: enable HSE, PLL  (not implemented)
//=================================================================================================

void SYSCLKConfig_STOP(void)
{
  /* After wake-up from STOP reconfigure the system clock */
	  /* Disable HSE */
		RCC_DeInit();
		RCC_HSEConfig(RCC_HSE_OFF);
		RCC_PLLCmd(DISABLE);
}

//=================================================================================================
//                               Reset GPIOs MODER registers.
//=================================================================================================

void Init_Reset_GPIOs(void)
{
  // Enable GPIO clocks
  RCC->AHBENR |= (RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOBEN | RCC_AHBENR_GPIOCEN | RCC_AHBENR_GPIODEN | RCC_AHBENR_GPIOFEN);

  GPIOA->MODER = 0;
  GPIOB->MODER = 0;
  GPIOC->MODER = 0;
  GPIOD->MODER = 0;
  GPIOF->MODER = 0;
}

//=================================================================================================
//            Configure GPIOs in "Low Power mode" : Analog Mode, No Pull-Up nor Pull-Down
//=================================================================================================
//#if USE_LPM > 0
void Init_LPM_GPIOs(void)
{
  // Enable GPIO clocks
  RCC->AHBENR |= (RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOBEN | RCC_AHBENR_GPIOCEN | RCC_AHBENR_GPIODEN | RCC_AHBENR_GPIOFEN);

  GPIOA->MODER |= 0xFFFCFFFF; // If PA8 is used (DEBUG_IO)
  //GPIOA->MODER |= 0xFFFFFFFF; // If PA8 is not used (DEBUG_IO)
  GPIOA->PUPDR = 0;

  GPIOB->MODER = 0xFFFFFFFF;
  GPIOB->PUPDR = 0;

  GPIOC->MODER = 0xFFFFFFFF;
  GPIOC->PUPDR = 0;

  GPIOD->MODER = 0xFFFFFFFF;
  GPIOD->PUPDR = 0;

  GPIOF->MODER = 0xFFFFFFFF;
  GPIOF->PUPDR = 0;
}
//#endif // USE_LPM

//=================================================================================================
//                        In case of I2C error print and LCD message.
//=================================================================================================





void clock_init(){
   /* Start with HSI clock (internal 8mhz), divide by 2 and multiply by 10 to
   * get: 40Mhz
   * Enable PLL, wait till it's stable, then select it as system clock*/
  RCC_PLLConfig(RCC_PLLSource_HSI_Div2, RCC_PLLMul_10);
  RCC_PLLCmd(ENABLE);
  while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET) {}
  RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

  /* Set HCLK, PCLK1, and PCLK2 to SCLK (these are default */
  RCC_HCLKConfig(RCC_SYSCLK_Div1);
  RCC_PCLK1Config(RCC_HCLK_Div1);
  RCC_PCLK2Config(RCC_HCLK_Div1);

}

void SysTick_Handler(void)
{

    // 1ms systick intrerupt
	if (Time < 65000){ Time++;}

	if (Time <= 2000)
	{

	if (i2ctime <= 5000){ i2ctime++;}
	else {i2ctime=5000;}

    if (Time3 < 99000){ Time3++;}
    ADC_Configuration();

	Vdmppt[20]=readADC1(4);

	for (k1 = 1; k1 < 21; ++k1) {Vdmppt[k1-1]=Vdmppt[k1];}//Vdmppt 20msec circular buffer

#if test1!=1
	//--------------------
	if (Vdmppt[20]<=1700 && Vdmppt[19]<=1700 && Vdmppt[18]<=1700) //if less than 20.4V for 3ms consecutive.
	{
	duty1=1200;
	dmppt_LD = 0;
	dmppt_TPV = 0;

	uverror=1;react2=20;
	}
	else {if (duty1>0){duty1--;} else {uverror=0;}}
	//----------------------
#endif

    tempADC1[0]=tempADC1[0]+Vdmppt[20];

#if test1!=1
    //--------------------------------------

    tempH=(dmppt_LD & 1);
    if (tempH>0 && ld235==1 && DMPPTLD[1][3]!=2 && DMPPTLD[1][3]!=1 && overtemp<70) {LD2_ON;LD[2]=1;}  else {LD2_OFF;LD[2]=0;} //2
    tempH=(dmppt_LD & 2);
    if (tempH>0 && ld235==1 && DMPPTLD[2][3]!=2 && DMPPTLD[2][3]!=1 && overtemp<70) {LD3_ON;LD[3]=1;}  else {LD3_OFF;LD[3]=0;} //3
    tempH=(dmppt_LD & 4);
    if (tempH>0 && ld235==1 && DMPPTLD[4][3]!=2 && DMPPTLD[4][3]!=1 && overtemp<70) {LD5_ON;LD[5]=1;}  else {LD5_OFF;LD[5]=0;} //5


	//--------------------------------------

    tempH=(dmppt_LD & 8);
    if (tempH>0 && ld146==1 && DMPPTLD[8][3]!=2 && DMPPTLD[8][3]!=1 && overtemp<70) {LD4_ON;LD[4]=1;}  else {LD4_OFF;LD[4]=0;} //4
    tempH=(dmppt_LD & 16);
    if (tempH>0 && ld146==1 && DMPPTLD[16][3]!=2 && DMPPTLD[16][3]!=1 && overtemp<70) {LD6_ON;LD[6]=1;}  else {LD6_OFF;LD[6]=0;} //6
    tempH=(dmppt_LD & 32);
    if (tempH>0 && ld146==1 && DMPPTLD[32][3]!=2 && DMPPTLD[32][3]!=1 && overtemp<70) {LD1_ON;LD[1]=1;}  else {LD1_OFF;LD[1]=0;} //1

    //--------------------------------------
if (dmppt_rx2[5]==1 && overtemp<70)
{
    tempH=(dmppt_TPV & 1);
    if (tempH>0) {BPV2_ON;TPVs[2]=1;}  else {BPV2_OFF;TPVs[2]=0;} //TPV2
    tempH=(dmppt_TPV & 2);
    if (tempH>0) {BPV3_ON;TPVs[3]=1;}  else {BPV3_OFF;TPVs[3]=0;} //TPV3
    tempH=(dmppt_TPV & 4);
    if (tempH>0) {BPV1_ON;TPVs[1]=1;}  else {BPV1_OFF;TPVs[1]=0;} //TPV1
}
else {BPV1_OFF;BPV2_OFF;BPV3_OFF;TPVs[1]=0;TPVs[2]=0;TPVs[3]=0;}

	//--------------------------------------

if (dmppt_rx2[6]==1 && overtemp<70)
{
    tempH=(dmppt_TPV & 8);
    if (tempH>0) {BPV5_ON;TPVs[5]=1;}  else {BPV5_OFF;TPVs[5]=0;} //TPV5
    tempH=(dmppt_TPV & 16);
    if (tempH>0) {BPV4_ON;TPVs[4]=1;}  else {BPV4_OFF;TPVs[4]=0;} //TPV4
    tempH=(dmppt_TPV & 32);
    if (tempH>0) {BPV6_ON;TPVs[6]=1;}  else {BPV6_OFF;TPVs[6]=0;} //TPV6
}
else {BPV4_OFF;BPV5_OFF;BPV6_OFF;TPVs[5]=0;TPVs[4]=0;TPVs[6]=0;}

    //---------------------------------------

#endif

    ADC_TempSensorVrefintCmd(ENABLE);
    tempADC1[1]=readADC1(5)+tempADC1[1];  // EXT Temp1
    tempADC1[2]=readADC1(6)+tempADC1[2];  // EXT Temp2
    tempADC1[3]=readADC1(16)+tempADC1[3]; // Internal Temp



	}

	iyi++;

}
